local RaidAchFilter = LibStub("AceAddon-3.0"):NewAddon("RaidAchFilter", "AceConsole-3.0")
local RAchFLDB = LibStub("LibDataBroker-1.1"):NewDataObject("RAchF", {
	type = "data source",
	text = "RaidAchievementFilter",
	icon = "Interface\\Icons\\Achievement_Dungeon_GloryoftheRaider",
	OnClick = function(self, button)
		if button == "RightButton" then
			InterfaceOptionsFrame_OpenToCategory("RaidAchieveFilter")
		else
			RaidAchFilter:Interact()
		end
	end,
	OnTooltipShow = function(tt)
		tt:AddLine("RaidAchievementFilter")
		tt:AddLine("|cffffffff" .. "Click to Toggle, Right-click for Options.")
	end,
})
local icon = LibStub("LibDBIcon-1.0")

function RaidAchFilter:ToggleLDBIcon()
	RaidAchFilter.db.profile.minimap.hide = not RaidAchFilter.db.profile.minimap.hide
	if RaidAchFilter.db.profile.minimap.hide then
		icon:Hide("RAchF")
	else
		icon:Show("RAchF")
	end
end

function RaidAchFilter:RegisterOptions()
	local AceConfig = LibStub("AceConfig-3.0")
	AceConfig:RegisterOptionsTable("RaidAchieveFilter", {
		type = 'group',
		args = {
			togglebutton = {
				type = 'toggle',
				order = 1,
				name = "Show minimap icon",
				desc = "Show/hide minimap icon.",
				get = function()
					return not RaidAchFilter.db.profile.minimap.hide
				end,
				set = RaidAchFilter.ToggleLDBIcon,
			},
		},
	})
	LibStub("AceConfigDialog-3.0"):AddToBlizOptions("RaidAchieveFilter")
end

RaidAchFilter:RegisterChatCommand("rachf", "Interact")
function RaidAchFilter:Interact()
	local frame = _G["RaidAch_Frame"]
	if RAF_Shown == nil or RAF_Shown == 0 then
		RAF_Shown = 1
		if frame == nil then RaidAch_Initialize(); ShowAch(1); RAF_RID = 1; else frame:Show() end
	else RAF_Shown = 0; frame:Hide() end
end

function RaidAchFilter:OnInitialize()
	self.db = LibStub("AceDB-3.0"):New("RAchF_DB", {
		profile = {
			minimap = {
				hide = false,
			},
		},
	})
	icon:Register("RAchF", RAchFLDB, self.db.profile.minimap)
	RaidAchFilter:RegisterOptions()
end

function RaidAch_Initialize()

	local frame = CreateFrame("Frame", "RaidAch_Frame", UIParent)

--	frame:RegisterEvent("ACHIEVEMENT_EARNED");
--	function frame:OnEvent(event, arg1)
--		for i = 1, #RList do
--			if RList[i] == arg1 then ShowAch(RAF_RID) end
--		end
--	end

	frame:SetFrameStrata("DIALOG")
	frame:SetMovable(true)
	frame:EnableMouse(true)
	frame:RegisterForDrag("LeftButton")
	frame:SetScript("OnDragStart", frame.StartMoving)
	frame:SetScript("OnDragStop", frame.StopMovingOrSizing)
	frame:SetPoint("CENTER"); 
	frame:SetWidth(520); frame:SetHeight(588);
	local tex = frame:CreateTexture("ARTWORK");
	tex:SetAllPoints();
	tex:SetTexture(0, 0, 0); tex:SetAlpha(0.8);
	frame:Show();

	local scrollframe = CreateFrame("ScrollFrame", "RaidAch_ScrollFrame", frame) 
	scrollframe:SetPoint("TOPLEFT", 10, -55) 
	scrollframe:SetPoint("BOTTOMRIGHT", -5, 10) 
	frame.scrollframe = scrollframe

	local scrollbar = CreateFrame("Slider", "RaidAch_Slider", scrollframe, "UIPanelScrollBarTemplate") 
	scrollbar:SetPoint("TOPLEFT", frame, "TOPRIGHT", 0, -16) 
	scrollbar:SetPoint("BOTTOMLEFT", frame, "BOTTOMRIGHT", 0, 16) 
	scrollbar:SetMinMaxValues(1, 100) 
	scrollbar:SetValueStep(50) 
	scrollbar.scrollStep = 50
	scrollbar:SetValue(0) 
	scrollbar:SetWidth(16) 
	scrollbar:SetScript("OnValueChanged", 
		function (self, value) self:GetParent():SetVerticalScroll(value) end) 
	
	local scrollbg = scrollbar:CreateTexture(nil, "BACKGROUND") 
	scrollbg:SetAllPoints(scrollbar) 
	scrollbg:SetTexture(0, 0, 0, 0.8) 
	frame.scrollbar = scrollbar 

	scrollframe:EnableMouseWheel(true)
	scrollframe:SetScript("OnMouseWheel",
		function(self, delta) 
			local x = scrollbar:GetValue(); local y = scrollbar:GetValueStep()*3
			if #RList > 7 then -- Only work if scrollbar is up
				if delta == 1 then scrollbar:SetValue(x-y)
				else scrollbar:SetValue(x+y) end 
			end
	end)

	local content = CreateFrame("Frame", "RaidAch_Content", scrollframe) 
	content:SetSize(128, 128) 
	scrollframe.content = content 
	scrollframe:SetScrollChild(content)

	-- Border
	local backdrop = {
	  edgeFile = "Interface\\Tooltips\\UI-Tooltip-Border",
	  tile = false, tileSize = 8, edgeSize = 8,
	  insets = { left = 2, right = 2, top = 2, bottom = 2 }
	}
	frame:SetBackdrop(backdrop)
	
	-- Header Text
	local ttl = frame:CreateFontString("$parent_HeaderText","OVERLAY","GameFontNormal")
	ttl:SetPoint("TOPLEFT",frame,"TOPLEFT",8,-5);
	ttl:SetText("Raid Achievement Filter");
	ttl:Show();

	-- Close Button
	local btn = CreateFrame("BUTTON", nil, frame, "UIPanelCloseButton")
	btn:SetPoint("TOPRIGHT", frame, "TOPRIGHT", 5, 5);
	btn:SetAlpha(1);
	btn:Show();
	btn:Enable();

	-- Hide Completed button
	local btn2 = CreateFrame("CheckButton", "$parent_Option", frame, "ChatConfigCheckButtonTemplate")
	btn2:SetPoint("TOPRIGHT", -105, -25);
	_G[btn2:GetName() .. "Text"]:SetText("Hide Completed")
	btn2:RegisterForClicks("AnyUp");
	btn2:SetScript("OnClick", function (self, button, down) 
	  if RAF_RID ~= nil then ShowAch(RAF_RID); end
	end)

	-- Raids list dropdown
	local dropDown = CreateFrame("Button", "RaidAch_Dropdown", frame, "UIDropDownMenuTemplate")
	UIDropDownMenu_Initialize(dropDown)
	UIDropDownMenu_SetText(dropDown, "Filter by Raid Instance")
	UIDropDownMenu_JustifyText(dropDown, "LEFT") 
	dropDown:SetPoint("TOPLEFT", frame, "TOPLEFT", -10, -20);
	UIDropDownMenu_SetWidth(dropDown, 200)	dropDown:SetAlpha(1);
	dropDown:SetScript("OnClick", function() 
		ToggleDropDownMenu(1, nil, dropDown, dropDown, 16, 5)
	end)

	dropDown.HideMenu = function()
	    if UIDROPDOWNMENU_OPEN_MENU == dropDown then
		CloseDropDownMenus()
	    end
	end

	local info = {}
	dropDown.initialize = function(self, level)
	    if not level then return end
		wipe(info)
		if level == 1 then
			info.isTitle = 1
			info.notCheckable = 1
			info.keepShownOnClick = false
			info.text = "Filter by Raid Instance"
			UIDropDownMenu_AddButton(info, level)

			info.disabled = nil
			info.isTitle = nil
			info.notCheckable = true
			info.hasArrow = true

			info.text = "Lich King"
			info.value = "submenu1"
			UIDropDownMenu_AddButton(info, level)
			info.text = "Cataclysm"
			info.value = "submenu2"
			UIDropDownMenu_AddButton(info, level)
			info.text = "Mists of Pandaria"
			info.value = "submenu3"
			UIDropDownMenu_AddButton(info, level)
			
			-- Close menu item
			info.hasArrow     = nil
			info.value        = nil
			info.notCheckable = 1
			info.text         = CLOSE
			info.func         = self.HideMenu
			UIDropDownMenu_AddButton(info, level)

		 elseif level == 2 then
		 
			dd_Array = {"Malygos (10/25 Player)","Sartharion (10 Player)","Sartharion (25 Player)","Naxx (10 Player)","Naxx (25 Player)","Ulduar (10 Player)",
			"Ulduar (25 Player)", "Onyxia's Lair (10/25 Player)", "Trial of the Crusader (10 Player)","Trial of the Crusader (25 Player)","Icecrown Citadel (10 Player)",
			"Icecrown Citadel (25 Player)","Halion (10/25 Player)", "Blackwing Descent","The Bastion of Twilight","Throne of the Four Winds","Firelands","Dragon Soul",
			"Heart of Fear","Mogu'shan Vaults","Terrace of Endless Spring"}
			if UIDROPDOWNMENU_MENU_VALUE == "submenu1" then
				for i = 1, 13 do
				  info.value = i
				  info.checked = false
				  info.text = dd_Array[i]
				  info.func = function() 
					UIDropDownMenu_SetText(dropDown, dd_Array[i]); ShowAch(i); RAF_RID = i; UIDropDownMenu_SetSelectedValue(dropDown, i); ToggleDropDownMenu(1, nil, dropDown);
				  end
				  UIDropDownMenu_AddButton(info, level)
				end
			elseif UIDROPDOWNMENU_MENU_VALUE == "submenu2" then
				for i = 14, 18 do
				  info.value = i
				  info.checked = false
				  info.text = dd_Array[i]
				  info.func = function() 
					UIDropDownMenu_SetText(dropDown, dd_Array[i]); ShowAch(i); RAF_RID = i; UIDropDownMenu_SetSelectedValue(dropDown, i);  ToggleDropDownMenu(1, nil, dropDown);
				end
				  UIDropDownMenu_AddButton(info, level)
				end
			elseif UIDROPDOWNMENU_MENU_VALUE == "submenu3" then
				for i = 19, 21 do
				  info.value = i
				  info.checked = false
				  info.text = dd_Array[i]
				  info.func = function() 
					UIDropDownMenu_SetText(dropDown, dd_Array[i]); ShowAch(i); RAF_RID = i; UIDropDownMenu_SetSelectedValue(dropDown, i);  ToggleDropDownMenu(1, nil, dropDown);
				  end
				  UIDropDownMenu_AddButton(info, level)
				end
			end
	     end
	 end

	--Set default when opened the first time
	UIDropDownMenu_Initialize(dropDown)
	UIDropDownMenu_SetSelectedValue(dropDown, 1);
	UIDropDownMenu_SetText(dropDown, "Filter by Raid Instance")

end

function ach_Add(fr_name, ach_ID, indent_num)

	local IDNumber, Name, Points, Completed, Month, Day, Year, Description, Flags, Image, RewardText, isGuildAch = GetAchievementInfo(ach_ID)
	local frame = _G["RaidAch_Content"]
	
	local fra = _G[fr_name];
	if fra == nil then fra = CreateFrame("Frame", fr_name, frame); fra.texture = fra:CreateTexture(); end
	fra:SetSize(500,70)
	fra:SetPoint("TOPLEFT", frame, "TOPLEFT", 0, indent_num);
	fra.texture:SetAllPoints(fra)
	if Completed == true then 
		fra.texture:SetTexture("Interface\\AchievementFrame\\UI-Achievement-AchievementBackground", "BACKGROUND")
		fra.texture:SetTexCoord(0, 1, 0, 0.5);
		fra:SetScript("OnMouseDown", function(self, button) if button == "LeftButton" then ChatEdit_InsertLink(GetAchievementLink(ach_ID)); end; end) -- clear the AchTracked stuff
	elseif Completed == false then
		fra.texture:SetTexture("Interface\\AchievementFrame\\UI-Achievement-Parchment-Horizontal-Desaturated", "BACKGROUND")
		fra.texture:SetTexCoord(0, 1, 0, 0.5);
		fra:SetScript("OnMouseDown", function(self, button) 
		  if button == "LeftButton" then ChatEdit_InsertLink(GetAchievementLink(ach_ID)); 
		  elseif button == "RightButton" then AddTrackedAchievement(ach_ID); end; 
		end)
	end
	fra:SetScript("OnDragStart", frame.StartMoving)
	fra:SetScript("OnDragStop", frame.StopMovingOrSizing)
	fra:Show()

	local fraicon = _G[fr_name .. "icon"]
	if fraicon == nil then 
		fraicon = CreateFrame("Frame",fr_name .. "icon",fra) 
		fraicon.texture = fraicon:CreateTexture()
	end
	fraicon:SetSize(45,45)
	fraicon:SetPoint("TOPLEFT",fr_name,"TOPLEFT",2,-2)
	fraicon.texture:SetTexture(Image, "ARTWORK")
	fraicon.texture:SetAllPoints(fraicon)
	fraicon.texture:SetTexCoord(0,1,0,1)
	if Completed == true then fraicon.texture:SetVertexColor(1,1,1,1) else fraicon.texture:SetVertexColor(.55, .55, .55, 1) end
	fraicon:Show()

	local frashield = _G[fr_name .. "shield"]
	if frashield == nil then 
		frashield = CreateFrame("Frame",fr_name .. "shield",fra) 
		frashield.texture = frashield:CreateTexture()
	end
	frashield.texture:SetTexture("Interface\\AchievementFrame\\UI-Achievement-Shields", "ARTWORK")
	frashield:SetSize(65,65)
	frashield:SetPoint("TOPRIGHT",fr_name,"TOPRIGHT",-1,-2)
	frashield.texture:SetAllPoints(frashield)
	if Completed == true then frashield.texture:SetTexCoord(0, 0.5, 0, 0.5) else frashield.texture:SetTexCoord(0.5, 1, 0, 0.5) end
	frashield:Show()

	local pts = _G[fr_name .. "shield_Points"]
	if pts == nil then pts = frashield:CreateFontString(fr_name .. "shield_Points","OVERLAY","GameFontNormalLarge") end
	if Completed == true then pts:SetFontObject("GameFontNormalLarge") else pts:SetFontObject("GameFontDisableLarge") end
	pts:SetPoint("CENTER",frashield,"CENTER",0,3)
	pts:SetText(Points)
	pts:Show()

	local achnm = _G[fr_name .. "AchName"]
	if achnm == nil then achnm = fra:CreateFontString(fr_name .. "AchName","OVERLAY","GameTooltipHeaderText") end
	if Completed == true then achnm:SetFontObject("GameFontWhite") else achnm:SetFontObject("GameFontDisable") end
	achnm:SetPoint("TOPLEFT",fra,"TOPLEFT",54,-6)
	achnm:SetText(Name)
	achnm:Show()

	local achdsc = _G[fr_name .. "AchDesc"]
	if achdsc == nil then achdsc = fra:CreateFontString(fr_name .. "AchDesc","OVERLAY","GameFontBlackSmall") end
	achdsc:SetFontObject("AchievementDescriptionFont")
	--if Completed == true then achdsc:SetFontObject("AchievementDescriptionFont") else achdsc:SetFontObject("AchievementDescriptionFont") end
	achdsc:SetSize(375,80)
	achdsc:SetJustifyH("LEFT"); achdsc:SetJustifyV("TOP"); 
	achdsc:CanWordWrap(1); achdsc:SetMaxLines(3);
	achdsc:SetPoint("TOPLEFT",fra,"TOPLEFT",54,-22)
	achdsc:SetText(Description)
	achdsc:Show()

end

function ShowAch(Raid)

	if Raid == 1 then RList = {1869,1874,622,1870,1875,623} end -- Malygos
	if Raid == 2 then RList = {2047,624,1876,2049,2050,2051} end --Sarth10
	if Raid == 3 then RList = {2048,1877,625,2052,2053,2054} end --Sarth25
	if Raid == 4 then RList = {1997,1858,562,1996,2182,566,2176,568,1856,2178,2180,564,2146,572,2184,574,576,578} end --Naxx10
	if Raid == 5 then RList = {2140,1859,563,2139,2183,567,2177,569,1857,2179,2181,565,2147,573,2185,575,577,579} end --Naxx25
	if Raid == 6 then RList = {2886,2888,2890,2892,2894,2909,3097,2907,2905,2911,2913,2914,2915,3056,2927,2925,2930,2919,2923,2933,2937,2931,2934,3058,2945,2947,2940,2939,2941,2953,2955,2959,2951,3006,3076,2985,2979,2980,2982,3177,3178,3179,2961,2967,2969,2963,3182,3138,2989,3180,2971,2973,2977,2975,3176,2996,3181,3008,3009,3012,3015,3014,3157,3141,3158,3159,3003,3036} end--Ulduar10
	if Raid == 7 then RList = {2887,2889,2891,2893,2895,2910,3098,2908,2906,2912,2918,2916,2917,3057,2928,2926,2929,2921,2924,2935,2938,2932,2936,3059,2946,2948,2943,2942,2944,2954,2956,2960,2952,3007,3077,2984,3118,2981,2983,3185,3186,3187,2962,2968,2970,2965,3184,2995,3237,3189,2972,2974,2978,2976,3183,2997,3188,3010,3011,3013,3016,3017,3161,3162,3163,3164,3002,3037} end--Ulduar25
	if Raid == 8 then RList = {4404,4402,4403,4396,4407,4405,4406,4397} end --Onyxia10
	if Raid == 9 then RList = {3797,3936,3996,3798,3799,3800,3917,3918} end --ToC10
	if Raid == 10 then RList = {3937,3813,3997,3815,3816,3916,3812} end --Toc25
	if Raid == 11 then RList = {4534,4535,4536,4537,4531,4628,4577,4538,4578,4528,4629,4582,4539,4529,4630,4579,4580,4527,4631,4601,4581,4530,4532,4636,4583} end --ICC10
	if Raid == 12 then RList = {4610,4611,4612,4613,4604,4632,4615,4614,4616,4605,4633,4617,4618,4606,4634,4619,4620,4607,4635,4621,4622,4597,4608,4637,4584} end --ICC25
	if Raid == 13 then RList = {4817,4818,4815,4816} end --Halion
	if Raid == 14 then RList = {5306,5307,5310,5308,5309,4849,5094,5107,5108,5109,5115,5116,4842} end --BWD
	if Raid == 15 then RList = {5300,4852,5311,5312,5118,5117,5119,5120,5121,4850} end --BoT
	if Raid == 16 then RList = {5304,5305,5122,5123,4851} end --Throne
	if Raid == 17 then RList = {5821,5829,5810,5813,5830,5799,5855,5807,5808,5806,5809,5805,5804,5803,5802} end --Firelands
	if Raid == 18 then RList = {6174,6128,6129,6175,6084,6105,6133,6180,6109,6110,6111,6112,6113,6114,6115,6116,6177} end --DragonSoul
	if Raid == 19 then RList = {6937,6936,6553,6683,6518,6922,6725,6726,6727,6728,6729,6730,6718,6845} end --Heart of Fear
	if Raid == 20 then RList = {6823,6674,7056,6687,6686,6455,6719,6720,6721,6722,6723,6724,6458,6844} end--Mogu'shan Vaults
	if Raid == 21 then RList = {6717,6933,6824,6825,6731,6732,6733,6734,6689} end--Terrace of Endless Spring

	local chkbox = _G["RaidAch_Frame_Option"]
	local scrollbar = _G["RaidAch_Slider"]

	for i = 1, 68 do
		local fra = _G["ach_Frame" .. i]
		if fra ~= nil then fra:Hide(); end
	end

	local maxid = 1
	local indent = 0
	franum = 1
	for i = 1, #RList do
		local IDNumber, Name, Points, Completed = GetAchievementInfo(RList[i])
		if Completed == true and chkbox:GetChecked() == nil then 
			ach_Add("ach_Frame" .. maxid, RList[i], indent)
			maxid=maxid+1
			indent=indent-75
		elseif Completed == false then 
			ach_Add("ach_Frame" .. maxid, RList[i], indent)
			maxid=maxid+1
			indent=indent-75
		end
	end
	maxid=(maxid*75) -- 75px per achievement box
	if maxid > 600 then maxid = maxid - 595; scrollbar:Show(); scrollbar:Enable(); 
	else maxid = 100; scrollbar:Hide(); scrollbar:Disable(); end -- hide scrollbar if not needed
	scrollbar:SetMinMaxValues(1, maxid) 
	scrollbar:SetValueStep(15.0)
	scrollbar:SetValue(0)

end